package com.me.common.ecs;

import com.badlogic.gdx.utils.Array;

public abstract class EntitySystem extends BaseSystem {

    private long typeBits;

    protected Array<Entity> entities;

    public EntitySystem(Engine engine, Class<? extends Component>... components) {
        super(engine);
        this.typeBits = ComponentType.getMaskBits(components);
        this.entities = new Array<>(true, 16, Entity.class);
    }

    public Array<Entity> getEntities() {
        return entities;
    }

    public void process(float dt) {
        for (int i = 0, n = getEntities().size; i < n; i++) {
            processEntity(entities.get(i), dt);
        }
    }

    public abstract void processEntity(Entity entity, float dt);

    public boolean interestedIn(Entity entity) {
        return (entity.componentBits & typeBits) == typeBits;
    }

    protected void addEntity(Entity entity) {
        entities.add(entity);
    }

    protected void removeEntity(Entity entity) {
        entities.removeValue(entity, true);
    }

}
